/*******************************************************************************
***  This file is part of Dux Rei Nummariae.                                 ***
***                                                                          ***
***  Copyright (C) 2017, 2018, 2019, 2020, 2021                              ***
***  John Schneiderman <Licensing _AT_ Schneiderman _DOT_ me>                ***
***                                                                          ***
***  This program is free software: you can redistribute it and/or modify it ***
***  under the terms of the GNU Affero General Public License as published   ***
***  by the Free Software Foundation, either version 3 of the License, or    ***
***  (at your option) any later version.                                     ***
***                                                                          ***
***  This program is distributed in the hope that it will be useful, but     ***
***  WITHOUT ANY WARRANTY; without even the implied warranty of              ***
***  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.                    ***
***  See the GNU Affero General Public License for more details.             ***
***                                                                          ***
***  You should have received a copy of the GNU Affero General Public License***
***  along with this program. If not, see <http://www.gnu.org/licenses/>.    ***
*******************************************************************************/
#include "BudgetSource.h"
using ::QString;
using drn::budgeting::BudgetSource;

#include <QObject>
using ::QObject;
#include <QString>
using ::QString;

#include <ostream>
using std::ostream;
#include <utility>
using std::move;

#include <foundation/Error.h>
using drn::foundation::Error;


BudgetSource::BudgetSource() :
	name_{}
{}

BudgetSource::BudgetSource(QString name) :
	name_{move(name)}
{
	this->name_ = this->name_.trimmed();

	if (this->name_.isEmpty())
		throw Error{QObject::tr("The name of a budget item source cannot be empty.")};
}

bool BudgetSource::operator==(const BudgetSource& other) const
{
	return this->name_ == other.name_;
}

bool BudgetSource::operator<(const BudgetSource& other) const
{
	return this->name_ < other.name_;
}

const QString& BudgetSource::name() const
{
	return this->name_;
}
void BudgetSource::setName(const QString& name)
{
	auto trimmedName{name.trimmed()};

	if (trimmedName.isEmpty())
		throw Error{
			QObject::tr("Cannot change a budget item source name from '%1' to '%2'.")
				.arg(this->name_)
				.arg(name)
		};
	this->name_ = move(trimmedName);
}

bool BudgetSource::isEmpty() const
{
	return this->name_.isEmpty();
}

bool drn::budgeting::operator!=(const BudgetSource& lhs, const BudgetSource& rhs)
{
	return ! (lhs == rhs);
}

ostream& drn::budgeting::operator<<(ostream& o, const BudgetSource& bs)
{
	return o << "Name (" << bs.name().toStdString() << ')';
}

QString drn::budgeting::presentationText(const BudgetSource& bs)
{
	return bs.name();
}
