/*******************************************************************************
***  This file is part of Dux Rei Nummariae.                                 ***
***                                                                          ***
***  Copyright (C) 2017, 2018, 2019, 2020, 2021                              ***
***  John Schneiderman <Licensing _AT_ Schneiderman _DOT_ me>                ***
***                                                                          ***
***  This program is free software: you can redistribute it and/or modify it ***
***  under the terms of the GNU Affero General Public License as published   ***
***  by the Free Software Foundation, either version 3 of the License, or    ***
***  (at your option) any later version.                                     ***
***                                                                          ***
***  This program is distributed in the hope that it will be useful, but     ***
***  WITHOUT ANY WARRANTY; without even the implied warranty of              ***
***  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.                    ***
***  See the GNU Affero General Public License for more details.             ***
***                                                                          ***
***  You should have received a copy of the GNU Affero General Public License***
***  along with this program. If not, see <http://www.gnu.org/licenses/>.    ***
*******************************************************************************/
#include "DebtMap.h"
using drn::budgeting::Debt;
using drn::budgeting::DebtMap;

#include <QObject>
using ::QObject;

#include "BudgetingErrors.h"
using drn::budgeting::BudgetItemError;


void DebtMap::update(Debt updated)
{
	const auto& source{updated.source()};

	if ( ! this->has(source))
		throw BudgetItemError{source, QObject::tr("Cannot update the debt as it does not exist.")};

	if (this->at(source).borrowed().code() != updated.borrowed().code())
		throw BudgetItemError{
			source,
			QObject::tr("The amount borrowed currency cannot be changed once set.")
		};

	if (this->at(source).borrowed() != updated.borrowed())
		throw BudgetItemError{
			source,
			QObject::tr("The amount borrowed cannot be changed once set.")
		};
	BudgetedMoneyMap<Debt>::update(updated);
}
