/*******************************************************************************
***  This file is part of Dux Rei Nummariae.                                 ***
***                                                                          ***
***  Copyright (C) 2017, 2018, 2019, 2020, 2021                              ***
***  John Schneiderman <Licensing _AT_ Schneiderman _DOT_ me>                ***
***                                                                          ***
***  This program is free software: you can redistribute it and/or modify it ***
***  under the terms of the GNU Affero General Public License as published   ***
***  by the Free Software Foundation, either version 3 of the License, or    ***
***  (at your option) any later version.                                     ***
***                                                                          ***
***  This program is distributed in the hope that it will be useful, but     ***
***  WITHOUT ANY WARRANTY; without even the implied warranty of              ***
***  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.                    ***
***  See the GNU Affero General Public License for more details.             ***
***                                                                          ***
***  You should have received a copy of the GNU Affero General Public License***
***  along with this program. If not, see <http://www.gnu.org/licenses/>.    ***
*******************************************************************************/
#ifndef DRN_DESKTOP_UI_INTERNAL_BANKENTRYWIDGET_H_
#define DRN_DESKTOP_UI_INTERNAL_BANKENTRYWIDGET_H_

#include <QWidget>

#include <set>

#include <banking/Bank.h>
#include <foundation/Optional.hpp>
#include <foundation/Switch.h>
#include <ui_BankEntryWidget.h>


class QString;

namespace drn
{
namespace accounting
{

class AccountNumber;

}
namespace foundation
{

template<typename>
class ObserverPtr;

}
namespace desktop_ui
{
namespace internal
{

class BankEntryWidget : public QWidget, Ui::BankEntryWdgt
{
	Q_OBJECT

	foundation::Optional<banking::Bank> original_;
	foundation::Switch isDirty_;

public slots:
	void markDirty();

public:
	BankEntryWidget(
		foundation::Optional<banking::Bank> b,
		const foundation::ObserverPtr<::QWidget> parent
	);
	static const ::QString& newBankNameText();
	const foundation::Optional<banking::Bank>& original() const;
	::QString name() const;
	bool isClosed() const;
	bool isRemoved() const;
	bool isDirty() const;
	void setErrorMessage(const ::QString& error);
	void resetDirtyFlag();
	void saved(std::set<accounting::AccountNumber> numbers);
};

}}}

#endif
