/*******************************************************************************
***  This file is part of Dux Rei Nummariae.                                 ***
***                                                                          ***
***  Copyright (C) 2017, 2018, 2019, 2020, 2021                              ***
***  John Schneiderman <Licensing _AT_ Schneiderman _DOT_ me>                ***
***                                                                          ***
***  This program is free software: you can redistribute it and/or modify it ***
***  under the terms of the GNU Affero General Public License as published   ***
***  by the Free Software Foundation, either version 3 of the License, or    ***
***  (at your option) any later version.                                     ***
***                                                                          ***
***  This program is distributed in the hope that it will be useful, but     ***
***  WITHOUT ANY WARRANTY; without even the implied warranty of              ***
***  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.                    ***
***  See the GNU Affero General Public License for more details.             ***
***                                                                          ***
***  You should have received a copy of the GNU Affero General Public License***
***  along with this program. If not, see <http://www.gnu.org/licenses/>.    ***
*******************************************************************************/
#include "DashboardWidget.h"
using ::QWidget;
using std::vector;
using drn::desktop_ui::internal::DashboardWidget;

#include <pecunia/Codes.h>
using pecunia::currency::Iso4217Codes;
#include <QtDebug>
// using qDebug

#include <adviser/BudgetHealth.h>
using drn::adviser::BudgetHealth;
#include <budgeting/Budget.h>
using drn::budgeting::BudgetTotals;
#include <budgeting/BudgetItemDue.h>
using drn::budgeting::BudgetItemDue;
#include "foundation/CalendarTime.hpp"
using drn::foundation::Days;
#include <foundation/ObserverPtr.hpp>
using drn::foundation::ObserverPtr;


DashboardWidget::DashboardWidget(
	const BudgetTotals& bt,
	const vector<BudgetItemDue>& due,
	const BudgetHealth& bh,
	const Days& dueWithin,
	const Iso4217Codes& preferred,
	const ObserverPtr<QWidget>& parent
) :
	QWidget{parent.observed()}
{
	this->setAttribute(Qt::WA_DeleteOnClose);
	this->setupUi(this);
	connect(
		this->budgetHealth,
		&BudgetHealthWidget::displayBudgetInterface,
		this,
		&DashboardWidget::displayBudgetInterface
	);
	this->budgetHealth->updateTotals(bt, preferred);
	this->itemsDue->updateDue(due, dueWithin);
	this->budgetHealth->updateHealth(bh);
}
