/*******************************************************************************
***  This file is part of Dux Rei Nummariae.                                 ***
***                                                                          ***
***  Copyright (C) 2017, 2018, 2019, 2020, 2021                              ***
***  John Schneiderman <Licensing _AT_ Schneiderman _DOT_ me>                ***
***                                                                          ***
***  This program is free software: you can redistribute it and/or modify it ***
***  under the terms of the GNU Affero General Public License as published   ***
***  by the Free Software Foundation, either version 3 of the License, or    ***
***  (at your option) any later version.                                     ***
***                                                                          ***
***  This program is distributed in the hope that it will be useful, but     ***
***  WITHOUT ANY WARRANTY; without even the implied warranty of              ***
***  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.                    ***
***  See the GNU Affero General Public License for more details.             ***
***                                                                          ***
***  You should have received a copy of the GNU Affero General Public License***
***  along with this program. If not, see <http://www.gnu.org/licenses/>.    ***
*******************************************************************************/
#ifndef DRN_DESKTOP_UI_INTERNAL_DISABLESIGNALSTEMPORARILY_HPP_
#define DRN_DESKTOP_UI_INTERNAL_DISABLESIGNALSTEMPORARILY_HPP_

#include <QObject>

#include <type_traits>

#include <foundation/ObserverPtr.hpp>
#include <foundation/ScopeExecutor.h>


namespace drn
{
namespace desktop_ui
{
namespace internal
{

template<typename ObjectType>
class DisableSignalsTemporarily : foundation::ScopeExecutor
{
	static_assert(
		std::is_base_of<::QObject, ObjectType>::value,
		"Disabling an object's signals must derrive from a Qt object."
	);

	const bool oldSignalState_;

public:
	explicit DisableSignalsTemporarily(const foundation::ObserverPtr<ObjectType>& toBlock);

};

}}}

template<typename ObjectType>
drn::desktop_ui::internal::DisableSignalsTemporarily<ObjectType>::DisableSignalsTemporarily(
	const foundation::ObserverPtr<ObjectType>& toBlock
) :
	foundation::ScopeExecutor{
		[toBlock, this] ()
		{
			toBlock->blockSignals(this->oldSignalState_);
		}
	},
	oldSignalState_{toBlock->blockSignals(true)}
{}

#endif
