/*******************************************************************************
***  This file is part of Dux Rei Nummariae.                                 ***
***                                                                          ***
***  Copyright (C) 2017, 2018, 2019, 2020, 2021                              ***
***  John Schneiderman <Licensing _AT_ Schneiderman _DOT_ me>                ***
***                                                                          ***
***  This program is free software: you can redistribute it and/or modify it ***
***  under the terms of the GNU Affero General Public License as published   ***
***  by the Free Software Foundation, either version 3 of the License, or    ***
***  (at your option) any later version.                                     ***
***                                                                          ***
***  This program is distributed in the hope that it will be useful, but     ***
***  WITHOUT ANY WARRANTY; without even the implied warranty of              ***
***  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.                    ***
***  See the GNU Affero General Public License for more details.             ***
***                                                                          ***
***  You should have received a copy of the GNU Affero General Public License***
***  along with this program. If not, see <http://www.gnu.org/licenses/>.    ***
*******************************************************************************/
#ifndef DRN_DESKTOP_UI_INTERNAL_NONTRACKSWIDGET_H_
#define DRN_DESKTOP_UI_INTERNAL_NONTRACKSWIDGET_H_

#include <pecunia/Codes.h>
#include <QString>
#include <QWidget>

#include <chrono>
#include <functional>
#include <set>

#include <foundation/CalendarTime.hpp>
#include <foundation/Optional.hpp>
#include <foundation/ObserverPtr.hpp>
#include <ui_NontracksWidget.h>


class QAbstractButton;
class QPushButton;

namespace drn
{
namespace budgeting
{

class BudgetSource;
struct Nontrack;
struct NontrackMap;

}
namespace foundation
{

template<typename>
class ObserverPtr;

}
namespace desktop_ui
{
namespace internal
{

class NontrackEntryWidget;

/**
 * A Qt Widget that displays all the sources of nontrack. This object deletes itself when it closes.
 */
class NontracksWidget : public ::QWidget, Ui::NontracksWdgt
{
	Q_OBJECT

	foundation::Optional<foundation::ObserverPtr<internal::NontrackEntryWidget>> findNontrack(
		const std::function<bool(const internal::NontrackEntryWidget&)>& isMatch
	);
	void addNontrack(const budgeting::Nontrack& nontrack, const bool isNew);
	bool anyNontrackDirty() const;

	const std::chrono::hours hrInWeek_;
	const foundation::Days dayInWeek_;
	const pecunia::currency::Iso4217Codes preferred_;
	const std::set<pecunia::currency::Iso4217Codes> usableCurrencies_;
	foundation::ObserverPtr<::QPushButton> addPshbttn_;

signals:
	/// @brief Triggered when a new nontrack is entered.
	void added(const budgeting::Nontrack& nontrack);
	/// @brief Triggered when the name of a nontrack changes.
	void renamed(const budgeting::BudgetSource& original, const budgeting::BudgetSource& changed);
	/// @brief Triggered when the values of an nontrack changes.
	void updated(const budgeting::Nontrack& nontrack);
	/// @brief Triggered when a source of nontrack is removed.
	void removed(const budgeting::BudgetSource& source);
	/// @brief Triggered when the widget is in the process of being closed.
	void closing();

private slots:
	void addEntry();
	void saveNontracks();
	void updateTotal();
	void onClicked(foundation::ObserverPtr<::QAbstractButton> button);

public slots:
	bool close();
	void onSaveSucceeded(const budgeting::BudgetSource& source);
	void onSaveFailed(
		const foundation::Optional<budgeting::BudgetSource>& original,
		const ::QString& errorMessage,
		const foundation::Optional<budgeting::BudgetSource>& displayed
	);

public:
	NontracksWidget(
		const budgeting::NontrackMap& nontracks,
		const std::chrono::hours& hrInWeek,
		const foundation::Days& dayInWeek,
		const pecunia::currency::Iso4217Codes& preferred,
		const std::set<pecunia::currency::Iso4217Codes>& usableCurrencies,
		const foundation::ObserverPtr<::QWidget>& parent
	);
};

}}}

#endif
