/*******************************************************************************
***  This file is part of Dux Rei Nummariae.                                 ***
***                                                                          ***
***  Copyright (C) 2017, 2018, 2019, 2020, 2021                              ***
***  John Schneiderman <Licensing _AT_ Schneiderman _DOT_ me>                ***
***                                                                          ***
***  This program is free software: you can redistribute it and/or modify it ***
***  under the terms of the GNU Affero General Public License as published   ***
***  by the Free Software Foundation, either version 3 of the License, or    ***
***  (at your option) any later version.                                     ***
***                                                                          ***
***  This program is distributed in the hope that it will be useful, but     ***
***  WITHOUT ANY WARRANTY; without even the implied warranty of              ***
***  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.                    ***
***  See the GNU Affero General Public License for more details.             ***
***                                                                          ***
***  You should have received a copy of the GNU Affero General Public License***
***  along with this program. If not, see <http://www.gnu.org/licenses/>.    ***
*******************************************************************************/
#ifndef DRN_FILE_STORAGE_INTERNAL_XMLBUDGETFILE_H_
#define DRN_FILE_STORAGE_INTERNAL_XMLBUDGETFILE_H_

#include <map>
#include <tuple>


class QIODevice;

namespace drn
{
namespace accounting
{

class AccountCode;
class AccountNumber;
class GeneralLedger;

}
namespace banking
{

enum class SupportedAccountTypes;
struct BankMap;
class BankName;

}
namespace budgeting
{

class Budget;
class BudgetSource;

}
namespace foundation
{

template<typename, typename>
class TypeIndexMap;

}
namespace file_storage
{
namespace internal
{

std::tuple<
	drn::accounting::GeneralLedger,
	drn::budgeting::Budget,
	foundation::TypeIndexMap<budgeting::BudgetSource, accounting::AccountCode>,
	banking::BankMap,
	std::map<
		banking::BankName,
		std::map<accounting::AccountNumber, banking::SupportedAccountTypes>
	>
>read(QIODevice& file);
void write(
	const drn::budgeting::Budget& b,
	const drn::accounting::GeneralLedger& gl,
	const banking::BankMap& banks,
	const std::map<
		banking::BankName,
		std::map<accounting::AccountNumber, banking::SupportedAccountTypes>
	>& bankAccountTypes,
	const foundation::TypeIndexMap<budgeting::BudgetSource, accounting::AccountCode>& budgetCodes,
	QIODevice& file,
	const bool shouldIndent = false
);

}}}

#endif
