/*******************************************************************************
***  This file is part of Dux Rei Nummariae.                                 ***
***                                                                          ***
***  Copyright (C) 2017, 2018, 2019, 2020, 2021                              ***
***  John Schneiderman <Licensing _AT_ Schneiderman _DOT_ me>                ***
***                                                                          ***
***  This program is free software: you can redistribute it and/or modify it ***
***  under the terms of the GNU Affero General Public License as published   ***
***  by the Free Software Foundation, either version 3 of the License, or    ***
***  (at your option) any later version.                                     ***
***                                                                          ***
***  This program is distributed in the hope that it will be useful, but     ***
***  WITHOUT ANY WARRANTY; without even the implied warranty of              ***
***  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.                    ***
***  See the GNU Affero General Public License for more details.             ***
***                                                                          ***
***  You should have received a copy of the GNU Affero General Public License***
***  along with this program. If not, see <http://www.gnu.org/licenses/>.    ***
*******************************************************************************/
#include "Switch.h"
using drn::foundation::Switch;
using drn::foundation::SwitchStates;

#include <ostream>
using std::ostream;


Switch::Switch(const SwitchStates state) :
	state_{state}
{}

Switch::Switch(const bool state) :
	Switch{state ? SwitchStates::On : SwitchStates::Off}
{}

const SwitchStates& Switch::state() const
{
	return this->state_;
}

Switch::operator bool() const
{
	return this->isOn();
}

bool Switch::isOn() const
{
	return this->state_ == SwitchStates::On;
}

void Switch::flipOff()
{
	this->state_ = SwitchStates::Off;
}

void Switch::toggle()
{
	if (this->state_ == SwitchStates::On)
		this->state_ = SwitchStates::Off;
	else
		this->state_ = SwitchStates::On;
}

void Switch::flipOn()
{
	this->state_ = SwitchStates::On;
}

ostream& drn::foundation::operator<<(ostream& o, const SwitchStates& ss)
{
	return o << '(' << (ss == SwitchStates::On ? "on" : "off") << ')';
}

ostream& drn::foundation::operator<<(ostream& o, const Switch& s)
{
	return o << "(State: " << s.state() << ')';
}
