/*******************************************************************************
***  This file is part of Dux Rei Nummariae.                                 ***
***                                                                          ***
***  Copyright (C) 2017, 2018, 2019, 2020, 2021                              ***
***  John Schneiderman <Licensing _AT_ Schneiderman _DOT_ me>                ***
***                                                                          ***
***  This program is free software: you can redistribute it and/or modify it ***
***  under the terms of the GNU Affero General Public License as published   ***
***  by the Free Software Foundation, either version 3 of the License, or    ***
***  (at your option) any later version.                                     ***
***                                                                          ***
***  This program is distributed in the hope that it will be useful, but     ***
***  WITHOUT ANY WARRANTY; without even the implied warranty of              ***
***  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.                    ***
***  See the GNU Affero General Public License for more details.             ***
***                                                                          ***
***  You should have received a copy of the GNU Affero General Public License***
***  along with this program. If not, see <http://www.gnu.org/licenses/>.    ***
*******************************************************************************/
#include "BudgetBankAccount.h"
using std::set;
using drn::banking::BankAccount;
using drn::navigation::BudgetBankAccount;

#include <algorithm>
using std::find_if;
#include <tuple>
using std::tie;

#include <budgeting/BudgetSource.h>
using drn::budgeting::BudgetSource;
#include <foundation/Optional.hpp>
using drn::foundation::Optional;


bool drn::navigation::operator<(const BudgetBankAccount& lhs, const BudgetBankAccount& rhs)
{
	return tie(lhs.id_, lhs.bankAccount_) < tie(rhs.id_, rhs.bankAccount_);
}

bool drn::navigation::operator==(const BudgetBankAccount& lhs, const BudgetBankAccount& rhs)
{
	return tie(lhs.id_, lhs.bankAccount_) == tie(rhs.id_, rhs.bankAccount_);
}

bool drn::navigation::operator!=(const BudgetBankAccount& lhs, const BudgetBankAccount& rhs)
{
	return ! (lhs == rhs);
}

Optional<BankAccount> drn::navigation::lookUpAccount(
	const BudgetSource& toFind,
	const set<BudgetBankAccount>& accounts
)
{
	const auto account{
		find_if(
			accounts.cbegin(),
			accounts.cend(),
			[&toFind] (const auto& bba)
			{
				return bba.id_.source_ == toFind;
			}
		)
	};

	if (account == accounts.cend())
		return {};
	return {account->bankAccount_};
}
